﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using MeOCRLib;                         // Step 1: Add MeOCR Lib to your project. Load meocrlib.dll reference and add the "using MeOCRLib" directive;

namespace CSharpEx
{
    public partial class frmCSharpEx_02 : Form
    {
        Puma ocrPuma;                  // Step 2: Declare your object variable

        public frmCSharpEx_02()
        {
            InitializeComponent();
            
            // Step 3: Create a new OCR engine instance preferrably at startup.
            ocrPuma = new Puma();

            // Step 4: Initialize the engine and pass the callback function.
            ocrPuma.Init(PumaProgress);

            // Step 5: Set the language.
            ocrPuma.Language = 0;      // 0 for English       

            // Step 6: Set the Output format.
            ocrPuma.OutputFormat = 2;  // Formatted text

            //Example of enumerating supported languages by populate a language drop down list.
            LoadLaguagesList();
        }

        private void btnRecognize_Click(object sender, EventArgs e)
        {
            //Set wait cursor
            Cursor.Current = Cursors.WaitCursor;

            //Get the image path. You may need to change this or just copy Sample_01.tif into the bin folder.
            string FileName = System.IO.Path.GetDirectoryName(Environment.GetCommandLineArgs()[0]) + @"\Sample_01.tif";

            try
            {

                //Load the image.
                Image imgPuma = Image.FromFile(FileName);

                // Step 7: Recognize the image.
                textOut.Text = ocrPuma.Recognize(imgPuma);

                //Demonstrates parsing of Raw Format
                if (ocrPuma.OutputFormat == 0)
                    ParseResults(textOut.Text);
            }
            catch(System.IO.FileNotFoundException)
            {
                MessageBox.Show(this, "File not found. [" + FileName + "] \n\nDid you forget to set the correct image path and make sure the image exists there?", this.Text);
            }

            //Restore default cursor
            Cursor.Current = Cursors.Default;
        }

        //-----------------------------------------------------------
        //Progess bar callback function
        //-----------------------------------------------------------
        //Does not get called when inside the designer (IDE). 
        //To see it operate correctly run the application outside the IDE.
        //-----------------------------------------------------------
        void PumaProgress(int step, string stepName, int lPercent)
        {
            btnRecognize.Text = lPercent + "% - " + stepName;
        }

        //-----------------------------------------------------------
        //Set Output Format
        //-----------------------------------------------------------
        private void rbRaw_CheckedChanged(object sender, EventArgs e)
        {
            btnRecognize.Text = "Click Here To Recognize";
            if (rbRaw.Checked)
                ocrPuma.OutputFormat = Convert.ToInt16(rbRaw.Tag);
        }

        private void rbPlain_CheckedChanged(object sender, EventArgs e)
        {
            btnRecognize.Text = "Click Here To Recognize";
            if (rbPlain.Checked)
                ocrPuma.OutputFormat = Convert.ToInt16(rbPlain.Tag);
        }

        private void rbFormatted_CheckedChanged(object sender, EventArgs e)
        {
            btnRecognize.Text = "Click Here To Recognize";
            if (rbFormatted.Checked)
                ocrPuma.OutputFormat = Convert.ToInt16(rbFormatted.Tag);
        }

        private void cmbLanguages_SelectedIndexChanged(object sender, EventArgs e)
        {
            //Set the language selected
            ocrPuma.Language = (short)cmbLanguages.SelectedIndex;  
        }

        //-----------------------------------------------------------
        //Enumerate Languages and load them into cmbLanguages. English is 0  
        //-----------------------------------------------------------
        void LoadLaguagesList()
        {
            short lIndex = 0;
            string langStr = ocrPuma.LanguageString(lIndex);
            while (langStr != "")
            {
                if (langStr != "Reserved")
                    cmbLanguages.Items.Add(langStr);
                lIndex++;
                langStr = ocrPuma.LanguageString(lIndex);
            }
            cmbLanguages.SelectedIndex = ocrPuma.Language;  
        }

        //-----------------------------------------------------------
        //Demonstrates parsing of Raw Format
        //-----------------------------------------------------------
        void ParseResults(string chars)
        {
           int numItems = -1;

            if (chars != "")
            {
                string[] charToks = chars.Split('\r');

                foreach (string Tok in charToks)
                {
                    if (numItems == -1)
                    {
                        numItems = Convert.ToInt32(Tok);
                    }
                    else
                    {

                        string[] TokData = Tok.Split('\x01');
                        if (TokData[0] != "")
                        {

                            switch (TokData[0][0])
                            {
                            	case ' ':           //Specifies the end of a word.
                               	    break;

                                case '\x02':	    //Specifies the end of a line fragment.	
                                    break;		
                            
                                default:    //OCR character
                                    char OCRChar = TokData[0][0];
                                    int Left     = Convert.ToInt32(TokData[1]);
                                    int Top      = Convert.ToInt32(TokData[2]);
                                    int Right    = Convert.ToInt32(TokData[3]);
                                    int Bottom   = Convert.ToInt32(TokData[4]);
                                    break;

                            }//switch (TokData[0][0])

                        }//if (TokData[0] != "")
                    
                    }//if (numItems == -1)


                }//foreach (string Tok in charToks)

            }//if (chars != "")
        }
    }
}
